#pragma once

#include "CANSD540Driver.h"

#include <string>

#include "Base.h"
#include "ErrorBase.h"

namespace mindsensors {

class CANSD540 : public frc::ErrorBase {
public:
    static std::string GetLibraryVersion();
    
    /**
     * An enumeration representing how the SD540C should stop.
     */
    enum StopMode : uint8_t {
        /**
         * Allow the motor to freely turn once stopped.
         */
        Coast = 0,
        /**
         * Motor resist motion when it is stopped.
         */
        Brake = 1
    };
    
    /**
     * An enumeration representing the default state of a limit switch.
     * @see {@link #EnableLimitSwitches()}
     * @see {@link #SetForwardLimitSwitch()}
     * @see {@link #SetReverseLimitSwitch()}
     */
    enum LimitSwitchBehavior : uint8_t {
        /**
         * This limit switch is expected to be normally open and will only activate when it becomes closed.
         */
        NormallyOpen = 0,
        /**
         * This limit switch is expected to be normally closed and will only activate when it becomes open.
         */
        NormallyClosed = 1
    };
    
	explicit CANSD540(uint8_t deviceNumber);

	    uint8_t GetDeviceID() const;
	std::string GetDeviceName() const;
	std::string GetFirmwareVersion() const;
	std::string GetHardwareVersion() const;
	std::string GetBootloaderVersion() const;
	std::string GetSerialNumber() const;

	void BlinkLED(uint8_t seconds);

    void Set(double speed);
    double Get() const;

    void SetInverted(bool isInverted);
    bool GetInverted() const;
    
    void SetStopMode(CANSD540::StopMode stopMode);
    CANSD540::StopMode GetStopMode() const;
    
    void EnableLimitSwitches(bool limitSwitchesEnabled);
    bool GetLimitSwitchesEnabled() const;

    void SetForwardLimitSwitch(CANSD540::LimitSwitchBehavior behavior);
    CANSD540::LimitSwitchBehavior GetForwardLimitSwitch() const;

    void SetReverseLimitSwitch(CANSD540::LimitSwitchBehavior behavior);
    CANSD540::LimitSwitchBehavior GetReverseLimitSwitch() const;
    
    void SetVoltageRamp(uint8_t voltageRamp);
    uint8_t GetVoltageRamp() const;
    
    double GetBatteryVoltage() const;
    
    bool GetForwardLimitSwitchState() const;
    bool GetReverseLimitSwitchState() const;

private:
	int m_handle;
};

} // namespace mindsensors
